@@ -7,6 +7,7 @@ import (
 	"net"
 
 	"github.com/metacubex/mihomo/component/ca"
+	"github.com/metacubex/mihomo/component/ech"
 	tlsC "github.com/metacubex/mihomo/component/tls"
 )
 
@@ -16,9 +17,14 @@ type TLSConfig struct {
 	FingerPrint       string
 	ClientFingerprint string
 	NextProtos        []string
+	ECH               *ech.Config
 	Reality           *tlsC.RealityConfig
 }
 
+type ECHConfig struct {
+	Enable bool
+}
+
 func StreamTLSConn(ctx context.Context, conn net.Conn, cfg *TLSConfig) (net.Conn, error) {
 	tlsConfig := &tls.Config{
 		ServerName:         cfg.Host,
@@ -33,8 +39,14 @@ func StreamTLSConn(ctx context.Context, conn net.Conn, cfg *TLSConfig) (net.Conn
 	}
 
 	if clientFingerprint, ok := tlsC.GetFingerprint(cfg.ClientFingerprint); ok {
+		tlsConfig := tlsC.UConfig(tlsConfig)
+		err = cfg.ECH.ClientHandle(ctx, tlsConfig)
+		if err != nil {
+			return nil, err
+		}
+
 		if cfg.Reality == nil {
-			tlsConn := tlsC.UClient(conn, tlsC.UConfig(tlsConfig), clientFingerprint)
+			tlsConn := tlsC.UClient(conn, tlsConfig, clientFingerprint)
 			err = tlsConn.HandshakeContext(ctx)
 			if err != nil {
 				return nil, err
@@ -48,6 +60,19 @@ func StreamTLSConn(ctx context.Context, conn net.Conn, cfg *TLSConfig) (net.Conn
 		return nil, errors.New("REALITY is based on uTLS, please set a client-fingerprint")
 	}
 
+	if cfg.ECH != nil {
+		tlsConfig := tlsC.UConfig(tlsConfig)
+		err = cfg.ECH.ClientHandle(ctx, tlsConfig)
+		if err != nil {
+			return nil, err
+		}
+
+		tlsConn := tlsC.Client(conn, tlsConfig)
+
+		err = tlsConn.HandshakeContext(ctx)
+		return tlsConn, err
+	}
+
 	tlsConn := tls.Client(conn, tlsConfig)
 
 	err = tlsConn.HandshakeContext(ctx)
